// GeoPolygon.cpp: implementation of the CGeoPolygon class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GeoPolygon.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CGeoPolygon, CObject, VERSIONABLE_SCHEMA | GEOIO_MODULE_VERSION_NUM)

CGeoPolygon::CGeoPolygon()
{
	m_nNumOfSides = 3;// by default its a triangle
	SetEntityType(GEOIO_POLYGON);
}

CGeoPolygon::CGeoPolygon(CGeoPolygon& pEntity)
{
	SetEntityType(pEntity.GetEntityType());
	SetEntityNumber(pEntity.GetEntityNumber());
	SetStartPoint(pEntity.GetStartPoint());
	SetEndPoint(pEntity.GetEndPoint());
	SetEntityColor(pEntity.GetEntityColor());
	m_nNumOfSides = pEntity.m_nNumOfSides;
}

CGeoPolygon::~CGeoPolygon()
{

}

void CGeoPolygon::Serialize(CArchive &ar)
{
	CPrimitive::Serialize(ar);

	if (ar.IsStoring())
	{	// storing code
		ar << m_nNumOfSides;
	}
	else
	{	// loading code
		ar >> m_nNumOfSides;
	}
}

/***********************************************************************************
		This fuction draws the polygon with m_StPnt as centre point and
		m_EndPnt as end point of the side of polygon.
		Pass the DC of the view class
***********************************************************************************/
void CGeoPolygon::Draw(CDC *pDC)
{		
	CPen pen(PS_SOLID, 0, m_clr);
	pDC->SelectObject(&pen);

	float pointX = m_EndPnt.x;
	float pointY = m_EndPnt.y;
	float centreX = m_StPnt.x;
	float centreY = m_StPnt.y;
	
	float phi = PI / ( m_nNumOfSides );	
	float teta = PI / 2 - phi;	
	float hypotenuse = sqrt( ( pointX - centreX ) * ( pointX - centreX ) + ( pointY - centreY ) * ( pointY - centreY ) );

	// draw the polygon
	hypotenuse = sqrt( ( pointX - centreX ) * ( pointX - centreX ) + ( pointY - centreY ) * ( pointY - centreY ) );
	for ( int f = 0;f < m_nNumOfSides;f ++ )
	{
		float x1 = hypotenuse * cos( teta );
		float mU = atan2( pointY - centreY , pointX - centreX );
		float x2 = x1 * cos( ( mU - teta ) );
		float y2 = x1 * sin( ( mU - teta ) );
		pDC->MoveTo( int( pointX ) , int( pointY ) );
		pDC->LineTo( int( pointX - 2 * x2 ) , int( pointY - 2 * y2 ) );
		pointX = int( pointX - 2 * x2 );
		pointY = int( pointY - 2 * y2 );
	}
}

/***********************************************************************************
		This fuction draws the polygon with m_StPnt as centre point and
		m_EndPnt as end point of the side of polygon.
		Pass the DC of the view class
***********************************************************************************/
void CGeoPolygon::Draw(CDC *pDC, COLORREF clr)
{		
	CPen pen(PS_SOLID, 0, clr);
	pDC->SelectObject(&pen);

	float pointX = m_EndPnt.x;
	float pointY = m_EndPnt.y;
	float centreX = m_StPnt.x;
	float centreY = m_StPnt.y;
	
	float phi = PI / ( m_nNumOfSides );	
	float teta = PI / 2 - phi;	
	float hypotenuse = sqrt( ( pointX - centreX ) * ( pointX - centreX ) + ( pointY - centreY ) * ( pointY - centreY ) );

	// draw the polygon
	hypotenuse = sqrt( ( pointX - centreX ) * ( pointX - centreX ) + ( pointY - centreY ) * ( pointY - centreY ) );
	for ( int f = 0;f < m_nNumOfSides;f ++ )
	{
		float x1 = hypotenuse * cos( teta );
		float mU = atan2( pointY - centreY , pointX - centreX );
		float x2 = x1 * cos( ( mU - teta ) );
		float y2 = x1 * sin( ( mU - teta ) );
		pDC->MoveTo( int( pointX ) , int( pointY ) );
		pDC->LineTo( int( pointX - 2 * x2 ) , int( pointY - 2 * y2 ) );
		pointX = int( pointX - 2 * x2 );
		pointY = int( pointY - 2 * y2 );
	}
}


void CGeoPolygon::DeleteEntity()
{
	SetDeleted();
}

BOOL CGeoPolygon::IsPointNearEntity(CPoint refPnt)
{
	return 1;
}

/*---------------------------------------------------------------------------------
	Function to mirror entity with respect to X-axis
---------------------------------------------------------------------------------*/
void CGeoPolygon::MirrorAboutXaxis()
{
	/* To mirror a polygon about X-axis, negate the start and 
	   end points' Ycoordinate*/
	m_StPnt.y = -(m_StPnt.y);
	m_EndPnt.y = -(m_EndPnt.y);
}

/*---------------------------------------------------------------------------------
	Function to mirror entity with respect to y-axis
---------------------------------------------------------------------------------*/
void CGeoPolygon::MirrorAboutYaxis()
{
	/* To mirror a polygon about Y-axis, negate the start and 
	   end points' Xcoordinate*/
	m_StPnt.x = -(m_StPnt.x);
	m_EndPnt.x = -(m_EndPnt.x);
}

/*---------------------------------------------------------------------------------
	Function to clone the entity. This function returns a new pointer to
	cloned entity.
---------------------------------------------------------------------------------*/
CPrimitive* CGeoPolygon::CreateClone()
{
	CGeoPolygon* pClone = new CGeoPolygon(*this);
	return pClone;
}