// LayerList.cpp: implementation of the CLayerList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "LayerList.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IMPLEMENT_SERIAL(CLayerList, CObject, VERSIONABLE_SCHEMA | GEOIO_MODULE_VERSION_NUM)

CLayerList::CLayerList()
{
	m_nLayerNumber = 1;
}

CLayerList::CLayerList(CLayerList& pLayerList)
{
	int nLayerCount = pLayerList.GetLayersCount();
	
	for( int ni=0; ni<nLayerCount; ni++ )
	{
		CLayer *pLayer = NULL,
			  *pCloneLayer = NULL;

		pLayer = (&pLayerList)->GetLayerAtIndex(ni);
		pCloneLayer = pLayer->CreateClone();
		
		// add the cloned entity to new layer created //
		AddLayerToList(pCloneLayer);
	}
}

CLayerList::~CLayerList()
{
	// delete all the layers in the layer list //
	while( !m_LayerList.IsEmpty() )
	{
		CLayer *pLayer = m_LayerList.GetHead();
		m_LayerList.RemoveHead();
		delete pLayer;
		pLayer = NULL;
	}
}

/******************************************************************************
	This function adds a new layer to the list
******************************************************************************/
void CLayerList::AddLayerToList(CLayer *pLayer)
{
	if(pLayer)
	{
		pLayer->SetLayerID(m_nLayerNumber);
		m_LayerList.AddTail(pLayer);
		m_nLayerNumber++;
	}
}


void CLayerList::Serialize(CArchive &ar)
{
	if (ar.IsStoring())
	{	// storing code
	}
	else
	{	// loading code
	}
	m_LayerList.Serialize(ar);
}

/******************************************************************************
	This function deletes all the layers in the list
******************************************************************************/
void CLayerList::DeleteAllLayers()
{
	if( m_LayerList.GetCount() > 0)
	while( m_LayerList.IsEmpty() == FALSE )
		delete m_LayerList.RemoveHead();
}


/******************************************************************************
	This function gets the count of layers in the list
******************************************************************************/
int CLayerList::GetLayersCount()
{
	return ( m_LayerList.GetCount() );
}


/**************************************************************************************
	this function adds the entity to the active layer
**************************************************************************************/
void CLayerList::AddEntityToActiveLayer(CPrimitive *pEntity)
{
	POSITION pos = m_LayerList.GetHeadPosition();
	BOOL bActiveLayer = FALSE;
	CLayer *pLayer = NULL;

	while( bActiveLayer == FALSE )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);

		bActiveLayer = pLayer->IsActiveLayer();
	}

	if( pLayer != NULL )
		pLayer->AddEntityToLayer(pEntity);
}

/**************************************************************************************
	this function draws the entities in the active alyer
**************************************************************************************/
void CLayerList::DrawActiveLayer(CDC *pDC)
{
	POSITION pos = m_LayerList.GetHeadPosition();
	if( pos == NULL )
		return;

	BOOL bActiveLayer = FALSE;
	CLayer *pLayer = NULL;

	while( bActiveLayer == FALSE )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);

		bActiveLayer = pLayer->IsActiveLayer();
	}

	if( pLayer != NULL )
		pLayer->DrawLayer(pDC);
}

/**************************************************************************************
	this function draws the entities of all the visible layers
**************************************************************************************/
void CLayerList::DrawAllVisibleLayers(CDC *pDC)
{
	POSITION pos = m_LayerList.GetHeadPosition();
	if( pos == NULL )
	{
		return;
	}

	CLayer *pLayer = NULL;

	while( NULL != pos )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);

		if(pLayer->IsLayerVisible())
		{
			pLayer->DrawLayer(pDC);
		}
	}
}


/**************************************************************************************
	this function gets the pointer to a layer at specified index... 
**************************************************************************************/
CLayer* CLayerList::GetLayerAtIndex(int nIndex)
{
	POSITION pos = m_LayerList.GetHeadPosition();
	CLayer *pLayer = NULL;

	if( nIndex == 1 )
	{
		pLayer = (CLayer *)m_LayerList.GetAt(pos);
		return pLayer;;
	}

	
	int nCount = m_LayerList.GetCount();
	int ni = 0;

	while( ni < nIndex )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);
		ni++;		
	}

	return pLayer;
}

/**************************************************************************************
	this function makes all the active layers inactive
**************************************************************************************/
void CLayerList::MakeAllLayersInactive()
{
	POSITION pos = m_LayerList.GetHeadPosition();
	CLayer *pLayer = NULL;
	int nCount = m_LayerList.GetCount();
	int ni = 0;

	while( ni < nCount )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);
		pLayer->SetAsInactiveLayer();
		ni++;		
	}
}

/**************************************************************************************
	this function removes the layer from the list at specified index....
**************************************************************************************/
BOOL CLayerList::RemoveLayerAtIndex(int nIndex)
{
	int nCount = m_LayerList.GetCount();
	if( nIndex > nCount )
		return FALSE;

	POSITION pos = m_LayerList.GetHeadPosition();
	POSITION prev = NULL;

	if( pos == NULL )
		return FALSE;

	CLayer * pLayer = (CLayer *)m_LayerList.GetAt(pos);
	int ni=0;
	while( ni <= nCount )
	{	
		pLayer = (CLayer *)m_LayerList.GetNext(pos);
		
		if( ni == nIndex-1 )
		{
			m_LayerList.RemoveAt(prev);
			return TRUE;
		}

		prev = pos;
		ni++;
		
	}


	return TRUE;
}

/**************************************************************************************
	this function returns the active layer
**************************************************************************************/
CLayer* CLayerList::GetActiveLayer()
{
	POSITION pos = m_LayerList.GetHeadPosition();
	if( pos == NULL )
		return NULL;

	BOOL bActiveLayer = FALSE;
	CLayer *pLayer = NULL;

	while( bActiveLayer == FALSE )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);

		bActiveLayer = pLayer->IsActiveLayer();
	}

	if( pLayer != NULL )
		return pLayer;

	return NULL;
}

/**************************************************************************************
	this function draws all the layers in the layer list
**************************************************************************************/
void CLayerList::DrawAllLayers(CDC *pDC)
{
	int nLayerCount = GetLayersCount();
	int ni = 1;
	while( ni <= nLayerCount )
	{
		CLayer *pLayer = GetLayerAtIndex( ni );
		pLayer->DrawLayer(pDC);
		ni++;
	}
}

/**************************************************************************************
	this function gets the last entity in the entity list for undo 
	operation
**************************************************************************************/
CPrimitive* CLayerList::GetEntityForUndoOperationInActiveLayer()
{
	POSITION pos = m_LayerList.GetHeadPosition();
	BOOL bActiveLayer = FALSE;
	CLayer *pLayer = NULL;
	CPrimitive *pEntity = NULL;


	while( bActiveLayer == FALSE )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);

		bActiveLayer = pLayer->IsActiveLayer();
		if( bActiveLayer )
		{
			pEntity = pLayer->GetEntityForUndoOperation();
			return pEntity;
		}
	}

	return pEntity;
}

/**************************************************************************************
	this function gets the last entity in the undo list for redo 
	operation
**************************************************************************************/
CPrimitive* CLayerList::GetEntityForRedoOperationInActiveLayer()
{
	POSITION pos = m_LayerList.GetHeadPosition();
	BOOL bActiveLayer = FALSE;
	CLayer *pLayer = NULL;
	CPrimitive *pEntity = NULL;


	while( bActiveLayer == FALSE )
	{
		pLayer = (CLayer *)m_LayerList.GetNext(pos);

		bActiveLayer = pLayer->IsActiveLayer();
		if( bActiveLayer )
		{
			pEntity = pLayer->GetEntityForRedoOperation();
			return pEntity;
		}
	}

	return pEntity;
}
/**************************************************************************************
	this function adds an entity in the entity list for undo 
	operation
**************************************************************************************/
void CLayerList::AddEntityToUndoList(CPrimitive *pEntity)
{
	CLayer *pLayer = GetActiveLayer();
	if( pLayer != NULL )
		pLayer->AddEntityToUndoList(pEntity);	

}

/**************************************************************************************
	this function gets the number of entities in the active layer
**************************************************************************************/
int CLayerList::GetEntityCountInActiveLayer()
{
	CLayer *pLayer = GetActiveLayer();
	if( pLayer == NULL )
		return 0;

	return pLayer->GetEntitiesCountInThisLayer();
}

/**************************************************************************************
	this function gets the number of entities in the undo list of
	active layer
**************************************************************************************/
int CLayerList::GetEntityCountInUndoList()
{
	CLayer *pLayer = GetActiveLayer();
	if( pLayer == NULL )
		return 0;

	return pLayer->GetEntityCountInUndoList();
}

/**************************************************************************************
	this function gets the previous layer w.r.t to the current(active) layer.
	If there is only one layer then pointer to this layer is returned.
**************************************************************************************/
CLayer* CLayerList::GetPreviousLayer()
{
	CLayer* pLayer = GetActiveLayer();
	if(GetLayersCount() == 1)
	{
		return pLayer;
	}

	return NULL;
}

/**************************************************************************************
	this function gets the next layer w.r.t to the current(active) layer.
	If there is only one layer then pointer to this layer is returned.
**************************************************************************************/
CLayer* CLayerList::GetNextLayer()
{
	CLayer* pLayer = GetActiveLayer();
	if(GetLayersCount() == 1)
	{
		return pLayer;
	}

	int nCurID = pLayer->GetLayerID();
	int nNextID = nCurID;

	POSITION pos = m_LayerList.GetHeadPosition();
	if( pos == NULL )
	{
		return NULL;
	}

	BOOL bContinue = TRUE;

	CLayer* pNextLayer = NULL;
	while( bContinue )
	{
		pNextLayer = (CLayer *)m_LayerList.GetAt(pos);
		
		nNextID = pNextLayer->GetLayerID();
		//bActiveLayer = pNextLayer->IsActiveLayer();

		if(nNextID > nCurID)
		{
			break;
		}
		else
		{
			pNextLayer = NULL;			
		}

		m_LayerList.GetNext(pos);
	}

	return pNextLayer;
}